/*********************************************************************************
**
**   $Id: reportgen.ui.h,v 1.3 2004/04/03 14:04:53 croftj Exp $
**   Copyright (c) 2004 Joe Croft <joe@croftj.net>
**   
**   This file is part of ah2.
**
**   ah2 is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   Foobar is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with Foobar; if not, write to the Free Software
**   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*********************************************************************************/

#include <qapplication.h>
#include <qcheckbox.h>
#include <qcursor.h>
#include <qdatetimeedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qprinter.h>
#include <qprocess.h>
#include <qregexp.h>
#include <qsettings.h>

extern QApplication *a;
extern QSettings Config;

void ReportGen::generateReport()
{
   QObject *wdt;
   QStringList cmdParms;
   QString p;
   QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
 
   printf("ReportGen::generateReport(): Enter\n");
   cmdParms.append(reportName);
   for (wdt = wdtList.first(); wdt; wdt = wdtList.next() ) 
   {
      printf("ReportGen::generateReport(): Have widget!\n");
      if ( wdt->isA("QComboBox") ) 
      {
         QRegExp re("\\((.*)\\)");
         p = QString(((QComboBox*)wdt)->currentText());
         if ( re.search(p) ) 
         {
            p = wdt->name() + QString("=") + re.cap(1);
         }
         cmdParms.append(p);
      }
      else if ( wdt->isA("QDateEdit") )
      {
         p = wdt->name() + QString("=") + QString(((QDateEdit*)wdt)->date().toString("MM/dd/yyyy"));
         cmdParms.append(p);
      }
      else if ( wdt->isA("QCheckBox") )
      {
         p = wdt->name() + QString("=");
         p += ( ((QCheckBox*)wdt)->isChecked() ) ? "Y" : "N";
         cmdParms.append(p);
      }
      else if ( wdt->isA("QLabel") )
      {
         p = wdt->name() + QString("=");
         p += ((QLabel*)wdt)->text();
         cmdParms.append(p);
      }
   }

   if ( ! do_preview ) 
   {
      QPrinter prt(QPrinter::HighResolution);
      prt.setFullPage(TRUE);
      prt.setColorMode(QPrinter::Color);
      prt.setPageSize(QPrinter::Letter);
      if ( prt.setup(this) )
      {
         p = QString("printer=") + prt.printerName();
         cmdParms.append(p);
         cmdParms.append("format=ps");
      }
   }

   QStringList::Iterator it;
   for (it = cmdParms.begin(); it != cmdParms.end(); it++) 
   {
      printf("Have parm '%s'\n", (const char*)(*it));
   }
   printf("ReportGen::generateReport(): Starting Process!\n");
   proc = new QProcess(cmdParms, this);
   a->connect(proc, SIGNAL(processExited()), this, SLOT(processComplete()));
   a->connect(proc, SIGNAL(readyReadStdout()), this, SLOT(readStdout()));

   p.setNum(db->port());
   QString dbString = db->databaseName() + ":" + db->hostName() + ":" + p + ":" +
                      db->userName() + ":" + db->password() + "\n\n\n";
//   printf("Sending |%s| to report\n", (const char*)dbString);
   haveProcess = true;
   if (proc->launch(dbString))
   {
      printf("Process started AOK!\n");
   }
   printf("ReportGen::generateReport(): Exit\n");
}

void ReportGen::printReport()
{
   haveProcess = false;
   do_preview = false;
   generateReport();
}

void ReportGen::previewReport()
{
   reportText->clear();
   reportString = "";
   haveProcess = false;
   do_preview = true;
   generateReport();
}

void ReportGen::setReport(QSqlDatabase *_db, QString *name)
{
   db = _db;

   haveProcess = false;
   QGridLayout *gl = new QGridLayout(controlFrame, 2, 2, 3);
   gl->setResizeMode(QLayout::Auto);

   reportName = *name;
   
   QFile rf(reportName);
   if ( rf.open(IO_ReadOnly) )
   {
      int row = 1;
      QString line;
      QRegExp re(QString("^#(title|arg):(.+):([a-z0-9]*):(date|selection|checkbox):(.*)$"));
      re.setMinimal(true);
      while (rf.readLine(line, 256) >= 0) 
      {
//         printf("Report line: |%s", (const char*)line);
         if (re.search(line) >= 0)
         {
            QString cmd       = re.cap(1);
            QString text      = re.cap(2);
            QString arg_name  = re.cap(3);
            QString ctl_type  = re.cap(4);
            QString options   = re.cap(5);
            printf("have cmd: |%s|\n", (const char*)cmd);
            if ( cmd == "title" ) 
            {
               QString title = text + QString(" Report");
//               setTitle(title);
               printf("Setting title to |%s|\n", (const char*)title);
               QLabel *l = new QLabel(title, controlFrame, "title");
               QFont f(l->font());
               f.setBold(true);
               f.setPointSize(f.pointSize() + 2);
               l->setFont(f);
               gl->addMultiCellWidget(l, 0, 0, 0, 1, Qt::AlignHCenter);
               wdtList.insert(wdtList.count(), l);
            }
            else if ( cmd == "arg" ) 
            {
               printf("have control type |%s|\n", (const char*)ctl_type);
               if ( ctl_type == "date" ) 
               {
                  printf("Adding date edit named |%s|\n", (const char*)arg_name);
                  QDateEdit *d = new QDateEdit(controlFrame, arg_name);
                  d->setDate(QDate::currentDate());
                  gl->addWidget(new QLabel(text, controlFrame), row, 0);
                  gl->addWidget(d, row, 1);
                  wdtList.insert(wdtList.count(), d);
               }
               else if ( ctl_type == "checkbox" )
               {
                  printf("Adding checkbox named |%s|\n", (const char*)arg_name);
                  QCheckBox *cb = new QCheckBox(text, controlFrame, arg_name);
                  gl->addMultiCellWidget(cb, row, row, 0, Qt::AlignLeft);
                  wdtList.insert(wdtList.count(), cb);
               }
               else if ( ctl_type == "selection" )
               {
                  printf("Adding selection named |%s|, options of |%s|\n", (const char*)arg_name, (const char*)options);
                  int ofs = 0;
                  int cnt = 0;
                  QComboBox *lb = new QComboBox(controlFrame, arg_name);
                  QRegExp re1("(.*)-(.*),");
                  re1.setMinimal(true);
                  while (re1.search(options, ofs) >= 0) 
                  {
                     cnt++;
                     QString opt = re1.cap(1);
                     QString name = re1.cap(2);
                     printf("found option of %s at ofs %d\n", (const char*)opt, ofs);
                     lb->insertItem( opt + QString(" (") + name + QString(")") );
                     ofs += re1.matchedLength();
                  }
                  gl->addWidget(new QLabel(text, controlFrame), row, 0);
                  gl->addWidget(lb, row, 1);
                  wdtList.insert(wdtList.count(), lb);
               }
            }
            row++;
         }
      }
      rf.close();
   }
}


void ReportGen::processComplete()
{
   printf("Process Completed!!!!\n");
   haveProcess = false;
   reportText->setText(reportString);
//   printf("Have report: %s\n", (const char *)reportString);
   delete proc;
   QApplication::restoreOverrideCursor();
}


void ReportGen::readStdout()
{
   while ( proc->canReadLineStdout() ) 
   {
      reportString += proc->readLineStdout();
   }
}




void ReportGen::done( int i)
{
   QApplication::restoreOverrideCursor();
   if ( haveProcess ) 
   {
      proc->tryTerminate();
   }
   else 
   {
      QDialog::done(i);
   }
}
